#include <stdio.h>
#include "../shmacsim_microbenchmark.h"

#define LOCK_ADDR       0x44000000
#define xStringify(s) stringify(s)
#define stringify(s) #s

ProcessorTask __processorTasks[SHMAC_MAX_GRIDDIM*SHMAC_MAX_GRIDDIM];
unsigned int __processorID;

// task function definitions
void testLockTime();

int main(int argc, char *argv[])
{	
    // initialization
    __processorID = SHMAC_PROCID;
    SHMAC_assignTasksToProcessors();
    // retrieve task for this processor
    ProcessorTask myTask = __processorTasks[__processorID];
    
    if(!myTask)
        printf("Processor %d was not assigned any tasks, exiting...\n", __processorID);   
    else
        myTask();
		
	return 0;
}

void testLockTime()
{
    // variables for time measurement
    unsigned int clkStart = 0, clkEnd = 0;
    printf("Processor %d: Test lock time...\n", __processorID);
    clkStart = SHMAC_CLKCNT;
    // ----- start of microbenchmark kernel -----
    // address of variable to lock
    __asm__ ("li $4," xStringify(LOCK_ADDR));
    // acquire lock
    __asm__ ( "initmutex:" );
    __asm__ ( "li $1, 1" );
    __asm__ ( "getmutex:" );
    __asm__ ( "ll $2, ($4)" );
    __asm__ ( "bne $zero, $2, getmutex" );
    __asm__ ( "sc $1, ($4)" );
    __asm__ ( "beq $zero, $1, initmutex" );
    // ----- end of microbenchmark kernel -----
    clkEnd = SHMAC_CLKCNT;
    
    // release lock
    volatile unsigned int * lockVar = (volatile unsigned int *) LOCK_ADDR;
    *lockVar = 0;
    
    // print statistics
    printf("Processor %d: Elapsed clock cycles: %d\n", __processorID, clkEnd-clkStart);
}


void SHMAC_assignTasksToProcessors()
{
    int i;
    // set all processor tasks
    for(i = 0; i < 16*16; i++)
        __processorTasks[i] = &testLockTime;
}


