#include <stdio.h>
#include "../shmacsim_microbenchmark.h"

#define REPEAT_COUNT 2000
#define REPEAT_COUNT_ASM ".rept 2000"

ProcessorTask __processorTasks[SHMAC_MAX_GRIDDIM*SHMAC_MAX_GRIDDIM];
unsigned int __processorID;

// perform write on neighbour if 1, perform read otherwise
int writeNeighbour; 

// task function definitions
void access11();

int main(int argc, char *argv[])
{	
    // initialization
    __processorID = SHMAC_PROCID;
    SHMAC_assignTasksToProcessors();
    // retrieve task for this processor
    ProcessorTask myTask = __processorTasks[__processorID];
    
    // benchmark-specific cmdline setup
    writeNeighbour = 0;
    if(argc > 0 && argv[0][0] == 'w')
        writeNeighbour = 1;
    
    if(!myTask)
        printf("Processor %d was not assigned any tasks, exiting...\n", __processorID);   
    else
        myTask();
		
	return 0;
}

void writeTo11()
{
    // variables for time measurement
    unsigned int clkStart = 0, clkEnd = 0;
    printf("Processor %d: Writing to (1,1) %d repeats...\n", __processorID, REPEAT_COUNT);
    clkStart = SHMAC_CLKCNT;
    // ----- start of microbenchmark kernel -----
    __asm__("li $4,0x11000000");
    __asm__(REPEAT_COUNT_ASM);
    __asm__("sw $5,0($4)");
    __asm__(".endr");
    // ----- end of microbenchmark kernel -----
    clkEnd = SHMAC_CLKCNT;
    
    // print statistics
    printf("Processor %d: Elapsed clock cycles: %d\n", __processorID, clkEnd-clkStart);
	printf("Processor %d: CPI: %d \n", __processorID, (clkEnd-clkStart) / REPEAT_COUNT);
}

void readFrom11()
{
    // variables for time measurement
    unsigned int clkStart = 0, clkEnd = 0;
    printf("Processor %d: Reading from (1,1) %d repeats...\n", __processorID, REPEAT_COUNT);
    clkStart = SHMAC_CLKCNT;
    // ----- start of microbenchmark kernel -----
    __asm__("li $4,0x11000000");
    __asm__(REPEAT_COUNT_ASM);
    __asm__("lw $5,0($4)");
    __asm__(".endr");
    // ----- end of microbenchmark kernel -----
    clkEnd = SHMAC_CLKCNT;
    
    // print statistics
    printf("Processor %d: Elapsed clock cycles: %d\n", __processorID, clkEnd-clkStart);
	printf("Processor %d: CPI: %d \n", __processorID, (clkEnd-clkStart) / REPEAT_COUNT);
}

void access11()
{
    // perform read or write depending on cmdline arg
    if(writeNeighbour)
        writeTo11();
    else
        readFrom11();
}

void SHMAC_assignTasksToProcessors()
{
    int i;
    // set all processor tasks
    for(i = 0; i < 16*16; i++)
        __processorTasks[i] = &access11;
}


