#include <stdio.h>
#include "../shmacsim_microbenchmark.h"

#define REPEAT_COUNT 2000
#define REPEAT_COUNT_ASM ".rept 2000"

// override clock count retrieval def using clock tile
// remove these two lines to use the system register instead
volatile unsigned int * clockTile = (volatile unsigned int *) 0x01000000;
#define SHMAC_CLKCNT    (*clockTile)

ProcessorTask __processorTasks[SHMAC_MAX_GRIDDIM*SHMAC_MAX_GRIDDIM];
unsigned int __processorID;

// task function definitions
void localInstrFetch();
void readFrom11();
void writeTo20();

int main(int argc, char *argv[])
{	
    // initialization
    __processorID = SHMAC_PROCID;
    SHMAC_assignTasksToProcessors();
    // retrieve task for this processor
    ProcessorTask myTask = __processorTasks[__processorID];
    
    if(!myTask)
        printf("Processor %d was not assigned any tasks, exiting...\n", __processorID);   
    else
        myTask();
		
	return 0;
}

void localInstrFetch()
{
    // variables for time measurement
    unsigned int clkStart = 0, clkEnd = 0;
    printf("Processor %d: Local memory read, %d repeats...\n", __processorID, REPEAT_COUNT);
    clkStart = SHMAC_CLKCNT;
    // ----- start of microbenchmark code -----
    __asm__("li $4,0");
    __asm__(REPEAT_COUNT_ASM);
    __asm__("addiu $4,$4,1");
    __asm__(".endr");
    // ----- end of microbenchmark code -----
    clkEnd = SHMAC_CLKCNT;
    
    // print statistics
    printf("Processor %d: Elapsed clock cycles: %d\n", __processorID, clkEnd-clkStart);
	printf("Processor %d: CPI: %d \n", __processorID, (clkEnd-clkStart) / REPEAT_COUNT);
}

void readFrom11()
{
    // variables for time measurement
    unsigned int clkStart = 0, clkEnd = 0;
    printf("Processor %d: Reading from (1,1) %d repeats...\n", __processorID, REPEAT_COUNT);
    clkStart = SHMAC_CLKCNT;
    // ----- start of microbenchmark kernel -----
    __asm__("li $4,0x11000000");
    __asm__(REPEAT_COUNT_ASM);
    __asm__("lw $5,0($4)");
    __asm__(".endr");
    // ----- end of microbenchmark kernel -----
    clkEnd = SHMAC_CLKCNT;
    
    // print statistics
    printf("Processor %d: Elapsed clock cycles: %d\n", __processorID, clkEnd-clkStart);
	printf("Processor %d: CPI: %d \n", __processorID, (clkEnd-clkStart) / REPEAT_COUNT);
}

void writeTo20()
{
    // variables for time measurement
    unsigned int clkStart = 0, clkEnd = 0;
    printf("Processor %d: Writing to (2,0) %d repeats...\n", __processorID, REPEAT_COUNT);
    clkStart = SHMAC_CLKCNT;
    // ----- start of microbenchmark kernel -----
    __asm__("li $4,0x20000000");
    __asm__(REPEAT_COUNT_ASM);
    __asm__("sw $5,0($4)");
    __asm__(".endr");
    // ----- end of microbenchmark kernel -----
    clkEnd = SHMAC_CLKCNT;
    
    // print statistics
    printf("Processor %d: Elapsed clock cycles: %d\n", __processorID, clkEnd-clkStart);
	printf("Processor %d: CPI: %d \n", __processorID, (clkEnd-clkStart) / REPEAT_COUNT);
}

void SHMAC_assignTasksToProcessors()
{
    int i;
    // reset all processor tasks
    for(i = 0; i < 16*16; i++)
        __processorTasks[i] = 0;
    // assign tasks
    __processorTasks[2] = &localInstrFetch;
    __processorTasks[16] = &readFrom11;
    __processorTasks[33] = &writeTo20;
}


