/*
 * SHMACGrid.h
 *
 *  Created on: 7 Mar 2013
 *      Author: Yaman Umuroglu
 */

#ifndef SHMACGRID_H_
#define SHMACGRID_H_

#include <vector>
#include "SHMACTile.h"

class SHMACGrid
{
public:
	SHMACGrid(uint8_t rowCount, uint8_t colCount);
	~SHMACGrid();

	// retrieve size of grid in i and j dimensions
	uint8_t getColCount() {return m_colCount;};
	uint8_t getRowCount() {return m_rowCount;};

	SHMACTile * get(uint8_t row, uint8_t col) const;
	SHMACTile * get(shmacsim::tile_coord coord) const;

	void set(uint8_t row, uint8_t col, SHMACTile * setTo);
	void set(shmacsim::tile_coord coord, SHMACTile * setTo);

	void establishConnections();
	void callRuntimeInitializers();

	static void connectNeighbors(SHMACTile *tile1, SHMACTile *tile2);
	static void connectGrid(SHMACGrid * grid, uint8_t rowCount, uint8_t colCount);

	friend ostream& operator<<(ostream& file, const SHMACGrid& dbv);
	SHMACTile *& operator() (const uint8_t row, const uint8_t col);

	// metadata setter functions for entire grid;
	// calls setter on each tile in grid
	void setMetaDataString(std::string key, std::string value);
	void setMetaDataInt(std::string key, unsigned int value);

protected:
	uint8_t m_colCount, m_rowCount;
	std::vector<std::vector<SHMACTile*> > m_elements;

};

ostream& operator<<(ostream& file, const SHMACGrid& dbv);

#endif /* SHMACGRID_H_ */
