/*
 * SHMACLibrary.cpp
 *
 *  Created on: 21 Mar 2013
 *      Author: Yaman Umuroglu
 */

#include "SHMACLibrary.h"
#include <iostream>

// weakly define the registerTileTypes function - will only be called
// if it is defined regularly somewhere else
extern void __registerTileTypes(SHMACLibrary *) __attribute__((weak));

SHMACLibrary::SHMACLibrary()
{
	if (__registerTileTypes)
	{
		__registerTileTypes(this);
		// TODO make debug output optional
		std::cerr << "SHMACLibrary registered " << m_entries.size()
				<< " tile types" << std::endl;
	}
}

bool SHMACLibrary::registerTileType(SHMACTileTypeID identifier,
		SHMACTileConstructFunction constructFunction)
{
	sc_assert(constructFunction);

	if (m_entries.find(identifier) != m_entries.end())
	{
		std::cerr << "SHMACLibrary::registerTileType: entry for tile type "
				<< identifier << " already exists!" << std::endl;
		return false;
	}

	m_entries[identifier] = constructFunction;

	return true;
}

bool SHMACLibrary::unregisterTileType(SHMACTileTypeID identifier)
{
	if (m_entries.find(identifier) == m_entries.end())
	{
		std::cerr << "SHMACLibrary::unregisterTileType: entry for tile type "
				<< identifier << " does not exist!" << std::endl;
		return false;
	}

	m_entries.erase(identifier);

	return true;
}

SHMACTile* SHMACLibrary::constructTile(SHMACTileTypeID identifier,
		shmacsim::tile_coord coord)
{
	if (m_entries.find(identifier) == m_entries.end())
	{
		std::cerr << "SHMACLibrary::constructTile: entry for tile type "
				<< identifier << " does not exist!" << std::endl;
		return NULL;
	}

	// call the tile constructor
	return (m_entries.at(identifier))(coord);
}
