/*
 * SHMACTile.h
 *
 *  Created on: 6 Mar 2013
 *      Author: Yaman Umuroglu
 */

#ifndef SHMACTILE_H_
#define SHMACTILE_H_

#include "SHMACElement.h"
#include "SHMACRouter.h"
#include "SHMACSlave.h"
#include "SHMACMaster.h"
#include "SHMACRouterBypass.h"	// for dual-port RAM support
#include <map>

class SHMACTile;

typedef void (*SHMACTileRuntimeInitializer)(SHMACTile * tile);

class SHMACTile: public SHMACElement
{
public:
	// any SHMAC elements passed to this constructor will have their
	// parent changed to this tile
	SHMACTile(shmacsim::tile_coord coords, SHMACRouter * router,
			SHMACMaster * master = NULL, SHMACSlave * slave = NULL);

	virtual ~SHMACTile();

	SHMACMaster* getMaster() const;
	SHMACRouter* getRouter() const;
	SHMACSlave* getSlave() const;

	// deduct neighborhood relationships between self & dest, connect to accordingly
	// only initializes ports from this->dest, NOT dest->this
	virtual void connectToTile(SHMACTile * dest);

	// runtime initializer configuration and execution:
	// set a function to be called right before simulation starts,
	// after the tile has been configured - a good place to do
	// extra configuration required for runtime
	void setRuntimeInitializer(SHMACTileRuntimeInitializer func);
	void executeRuntimeInitializer();

	// metadata setter-getter functions:
	// SHMACsim supports storing meta-data in each tile, either as integer
	// or string values, each associated with an integer key
	void setMetaDataString(std::string key, std::string value);
	std::string getMetaDataString(std::string key);
	void setMetaDataInt(std::string key, unsigned int value);
	unsigned int getMetaDataInt(std::string key);

protected:
	SHMACMaster * m_master;
	SHMACSlave * m_slave;
	SHMACRouter * m_router;
	SHMACRouterBypass * m_bypass;
	std::map<std::string, std::string> m_metaDataStr;
	std::map<std::string, unsigned int> m_metaDataInt;
	SHMACTileRuntimeInitializer m_runtimeInitializer;

	// change parentage and set connections between master-router-slave
	virtual void initializeTileInternalConnections();

};

#endif /* SHMACTILE_H_ */
