// shmac_simutils.h
// Various convenience and utility functions for SHMACsim

#ifndef SHMACSIM_SIMUTILS
#define SHMACSIM_SIMUTILS

#include <systemc.h>
#include <iostream>
#include "SHMACGrid.h"
#include "SHMACLibrary.h"

#define		SHMAC_CLOCK_PERIOD_NS	1

namespace shmacsim
{

// number of clock cycles in SystemC time
sc_time getClockCycles(unsigned int n);
// wait given number of clock cycles, to be used in SC_THREADs only
void waitClockCycles(unsigned int n);
// instantiate a SHMACGrid from a given config file
SHMACGrid * createGridFromConfigFile(const char * fileName);
// load tiles in a given SHMACGrid as described in the given config file
void configureGridFromConfigFile(const char * configFile, SHMACGrid * gr);
// returns direction from local tile to foreign tile
direction_enum directionOfCoords(tile_coord localCoords, tile_coord foreignCoords);
// convert between host and target system endianness
int32_t convertHostToTargetEndian_32bit(int32_t val);
// convert between tile_coord structure and 32 bit tile ID value
uint32_t procIDFromCoords(shmacsim::tile_coord coord);
shmacsim::tile_coord coordsFromProcID(uint32_t procID);
}

#endif	// SHMACSIM_SIMUTILS
