/*
 * shmac_types.cpp
 *
 * util functionality for types defined in shmac_types.h
 *
 *  Created on: 21 Feb 2013
 *      Author: Yaman Umuroglu
 */

#include "shmac_types.h"
#include "shmac_typeutils.h"

using namespace shmacsim;

std::string __directionNames[] = { "east", "north", "west", "south", "inproc",
		"inmem" };

direction_enum __directionInverse[] = { DIR_WEST, DIR_SOUTH, DIR_EAST,
		DIR_NORTH, DIR_INTERN_MEM, DIR_INTERN_PROC };

// operator implementations for shmacsim::tile_coord

ostream& operator<<(ostream& file, tile_coord const & dbv)
{
	return file << "(" << (int) dbv.i << ", " << (int) dbv.j << ")";
}

bool tile_coord::operator==(const tile_coord & rhs) const
{
	return rhs.i == i && rhs.j == j;
}

tile_coord& tile_coord::operator=(const tile_coord & rhs)
{
	i = rhs.i;
	j = rhs.j;

	return *this;
}

// operator implementations for shmacsim::packet

bool packet::operator==(const packet & rhs) const
{
	return rhs.address == address && rhs.data == data && rhs.flag == flag
			&& rhs.req == req && rhs.sender == sender
			&& rhs.write_enable == write_enable;
}

packet& packet::operator=(const packet & rhs)
{
	address = rhs.address;
	data = rhs.data;
	flag = rhs.flag;
	req = rhs.req;
	sender = rhs.sender;
	write_enable = rhs.write_enable;
	tag = rhs.tag;

	return *this;
}

// helper functions for SHMAC data types

tile_coord makeTileCoord(coord_int i, coord_int j)
{
	tile_coord newCoord;
	newCoord.i = i;
	newCoord.j = j;

	return newCoord;
}

packet makeRequestPacket(tile_coord sender, bus32 address, packet_flag sync,
		packet_wren wren)
{
// TODO speed improvement:
// this function is likely to be called many many times
// might be a good idea to not make any unnecessary copies of data
// on the stack, i.e send a pointer to an allocated packet instead
	packet newPacket;
// build a mem request packet
	newPacket.req = true;
	newPacket.address = address;
	newPacket.sender = sender;
	newPacket.data = 0; // no data for req
// set sync and write enable as given in parameters
	newPacket.flag = sync;
	newPacket.write_enable = wren;
	newPacket.tag = NULL;
	return newPacket;
}

tile_coord getTileCoordFromAddress(bus32 address)
{
	tile_coord newCoord;
	newCoord.i = (address >> SHMAC_I_SHIFT) & SHMAC_COORD_MASK;
	newCoord.j = (address >> SHMAC_J_SHIFT) & SHMAC_COORD_MASK;

	return newCoord;
}

bus32 setAddressTileCoord(bus32 oldAddress, tile_coord coord)
{
	return (coord.i << SHMAC_I_SHIFT) | (coord.j << SHMAC_J_SHIFT)
			| getLocalAddressFromAddress(oldAddress);
}

bus32 getLocalAddressFromAddress(bus32 address)
{
	return address & SHMAC_LADDR_MASK;
}

bus32 setAddressLocalAddress(bus32 oldAddress, bus32 newLocalAddress)
{
	return (oldAddress & (~SHMAC_LADDR_MASK))
			| (newLocalAddress & SHMAC_LADDR_MASK);
}
