/*
 * SHMACMasterArchCMIPS1.cpp
 *
 *  Created on: 6 Mar 2013
 *      Author: Yaman Umuroglu
 */

#include "SHMACMasterArchCMIPS1.h"
#include "shmac_typeutils.h"
#include "shmac_simutils.h"

SHMACMaster_ArchC_MIPS1::SHMACMaster_ArchC_MIPS1(sc_module_name name,
		SHMACElement* parent) :
		SHMACMaster(name, parent)
{
	// construct TLM adapter with name derived from master name
	char mod_name[20];
	sprintf(mod_name, "%s_ta", (const char *) name);
	m_tlmAdapter = new SHMACTLMAdapter(mod_name, parent);

	// construct sys regfile
	sprintf(mod_name, "%s_srf", (const char *) name);
	m_sysRegFile = new SHMACSysRegFile(mod_name);

	// construct MIPS processor
	sprintf(mod_name, "%s_proc", (const char *) name);
	m_proc = new mips1(mod_name);

	/* establish connections:
	 *
	 *  CPU core <-> TLM adapter <-> SysRegFile handler <-> in-out memports
	 *
	 */

	// bind processor TLM port <-> TLM adapter
	m_proc->DM_port.bind(*m_tlmAdapter);

	// bind TLM adapter outbound port -> sysregfile handler
	m_tlmAdapter->shmacMemPort.bind(*m_sysRegFile);

	// bind sysregfile router output port -> outbound mem port
	m_sysRegFile->routerPort.bind(outboundMemoryPort);

	// bind inbound memory port -> sysregfile handler
	inboundMemoryPort.bind(*m_sysRegFile);

	// bind sysregfile handler -> TLM adapter
	m_sysRegFile->cpuPort.bind(*m_tlmAdapter);
}

SHMACMaster_ArchC_MIPS1::~SHMACMaster_ArchC_MIPS1()
{
	wordfree(&m_wordexp);
	delete m_tlmAdapter;
	delete m_proc;
}

void SHMACMaster_ArchC_MIPS1::setParent(SHMACElement* parent)
{
	SHMACMaster::setParent(parent);
	// propagate parent changes to TLM adapter
	m_tlmAdapter->setParent(parent);
}

void SHMACMaster_ArchC_MIPS1::prepare(uint32_t heap_ptr, std::string cmdLine, bool integratedBootstrap)
{
	// FIXME need better local coordinate storage model
	m_sysRegFile->setTileID(shmacsim::procIDFromCoords(m_parent->getCoords()));
	// enable or disable integrated bootstrapping via sysregfile
	m_sysRegFile->setBootstrapEnabled(integratedBootstrap);

	// jump tile will take care of bootstrapping
	m_proc->set_ac_pc(0);
	m_proc->ac_start_addr = 0;

	// set up heap ptr, needed for syscall emulation
	m_proc->ac_heap_ptr = setAddressTileCoord(heap_ptr, m_parent->getCoords());

	// set "mem start" and "mem end" values
	// just here to keep that all instructions are fetched from local memory
	m_proc->ac_mem_start = ((unsigned int) setAddressTileCoord(0,
			m_parent->getCoords()));
	// FIXME mem_end should depend on available memory amount - not constant!
	m_proc->ac_mem_end = setAddressTileCoord(16 * 1024 * 1024 - 1,
			m_parent->getCoords());

	// parse cmd line arguments and pass on to ArchC core
	wordexp(cmdLine.c_str(), &m_wordexp, WRDE_SHOWERR);
	m_proc->set_args(m_wordexp.we_wordc, m_wordexp.we_wordv);

	m_proc->set_instr_batch_size(1);
	m_proc->init();
}

