/*
 * SHMACSysRegFileWithRAMPort.cpp
 *
 *  Created on: Jul 3, 2013
 *      Author: maltanar
 */

#include "SHMACRouterBypass.h"

SHMACRouterBypass::SHMACRouterBypass(sc_module_name name) : sc_module(name)
{
	m_localMemStart = m_localMemEnd = 0;
	m_bypassEnabled = false;
	// process received router msgs via receiveMessage
	fromRouterPort.bind(*this);
	// no more than 1 element should be needed
	// (since we can have a single outstanding req)
	m_bypassBuffer = new sc_fifo<shmacsim::packet>(5);

	SC_THREAD(sendBypassMessages);
}

SHMACRouterBypass::~SHMACRouterBypass()
{
	delete m_bypassBuffer;
}

void SHMACRouterBypass::setLocalMemRange(SHMAC_ADDR start, SHMAC_ADDR end)
{
	m_localMemEnd = end;
	m_localMemStart = start;
	m_bypassEnabled = true;
	std::cerr << "Bypass unit " << this->name() << " now has range " << std::hex << start << " to " << end << std::dec << std::endl;
}

void SHMACRouterBypass::receiveMessage(shmacsim::packet message)
{
	// the dual-port RAM with router bypass implementation is
	// all a big hack atm, should be properly done with a hw-like approach
	if(message.req)
		if((message.address >= m_localMemStart) && (message.address < m_localMemEnd) && m_bypassEnabled)
			// intercept packet & sent directly to dual port RAM instead of router
			m_bypassBuffer->write(message);
		else
			// regular request, send packet to router
			toRouterPort->receiveMessage(message);
	else
		// reply packet, send to master
		toMasterPort->receiveMessage(message);
}

void SHMACRouterBypass::sendBypassMessages()
{
	shmacsim::packet msg;
	while(1)
	{
		//wait(m_bypassBuffer->data_written_event());
		m_bypassBuffer->read(msg);
		toRAMPort->receiveMessage(msg);
	}
}
