#ifndef SHMACSYSREGFILE_H_
#define SHMACSYSREGFILE_H_

#include <systemc.h>
#include "shmac_types.h"
#include "shmac_interfaces.h"
#include "archc/ac_tlm_port.H"
#include "SHMACElement.h"

// note that min and max do not apply to bootstrapping addresses (0x0..0xc)
// which are handled by this version of the sysregfile
#define SHMAC_SYSREG_MIN	0xFFFFFDFF
#define SHMAC_SYSREG_CLK	(SHMAC_SYSREG_MIN + 0x0)	// clocks elapsed since sim start
#define SHMAC_SYSREG_TILEID	(SHMAC_SYSREG_MIN + 0x4)	// local tile ID

#define SHMAC_SYSREG_MAX	0xFFFFFEFF

class SHMACSysRegFile: public SHMACMemoryInterface, public sc_module
{
public:
	SHMACSysRegFile(sc_module_name name);

	sc_port<SHMACMemoryInterface> cpuPort, routerPort;

	// implemented from SHMACMemoryInterface
	// used to intercept memory requests from CPU to sys regfile regions
	// those requests are processed locally in this component
	// all other requests are forwarded to the router for normal processing
	virtual void receiveMessage(shmacsim::packet message);

	void setTileID(uint32_t tileID) {m_tileID = tileID;};
	uint32_t getTileID() {return m_tileID;};

	// enable or disable bootstrap packet handling
	void setBootstrapEnabled(bool enabled) {
		m_enableBootstrap = enabled;
	};

protected:
	void handleSysRegReqPacket(shmacsim::packet message);
	virtual void handleRegularPacket(shmacsim::packet message);
	// adapted from jump slave:
	// returns true if this packet is part of the bootstrapping process
	bool isBootstrap(shmacsim::packet message);

	// FIXME need better local coordinate storage solution
	uint32_t m_tileID;
	bool m_enableBootstrap;
};

#endif /* SHMACSYSREGFILE_H_ */
