/*
 * SHMACTLMAdapter.h
 *
 * adapter for converting between ArchC TLM memory IF <-> SHMAC memory IF
 *
 *  Created on: 4 Mar 2013
 *      Author: Yaman Umuroglu
 */

#ifndef SHMACTLMADAPTER_H_
#define SHMACTLMADAPTER_H_

#include <systemc.h>
#include "shmac_types.h"
#include "shmac_interfaces.h"
#include "archc/ac_tlm_port.H"
#include "SHMACElement.h"

class SHMACTLMAdapter: public ac_tlm_transport_if, public sc_module, public SHMACMemoryInterface {
public:
	// need to specify parent - will be used for source coord of outgoing packets
	SHMACTLMAdapter(sc_module_name name, SHMACElement * parent = NULL);

	sc_port<SHMACMemoryInterface> shmacMemPort;

	// implemented from ac_tlm_transport_if
	ac_tlm_rsp transport(const ac_tlm_req & req);

	// implemented from SHMACMemoryInterface
	void receiveMessage(shmacsim::packet message);

	SHMACElement* getParent() const
	{
		return m_parent;
	}

	void setParent(SHMACElement* parent)
	{
		m_parent = parent;
	}

protected:
	SHMACElement * m_parent;
	bool m_locked, m_scExecuted;
	shmacsim::bus32 m_lockAddress, m_lockReturnValue;

	// variables for the internal sync-async boundary
	bool m_pendingRequest;
	shmacsim::bus32 m_requestAddress;
	shmacsim::packet m_receivedData;
	sc_event m_replyReceived;

	virtual shmacsim::packet sendSHMACMessage(shmacsim::packet packet);
};

#endif /* SHMACTLMADAPTER_H_ */
