/*
 * SHMACTrafficTester.cpp
 *
 *  Created on: Jun 13, 2013
 *      Author: maltanar
 */

#include "SHMACTrafficTester.h"
#include "shmac_typeutils.h"

using namespace shmacsim;

SHMACTrafficTester::SHMACTrafficTester(sc_module_name instanceName, SHMACElement * parent) :
		SHMACMaster(instanceName, parent), m_delaySend(0, SC_NS), m_packetToSend(dummyPacket)
{
	m_sendPacketCount = 0;
	// get packets with local receiveMessage implementation
	inboundMemoryPort.bind(*this);

	// TODO add options to control:
	// - enable/disable packet generation
	// - enable/disable receive statistics
	SC_THREAD(generateTraffic);

	m_receivedPacketCount = 0;
	SC_THREAD(periodicReport);
}

SHMACTrafficTester::~SHMACTrafficTester()
{
	reportStats();
}

void SHMACTrafficTester::setSendPacket(shmacsim::packet pkt)
{
	m_packetToSend = pkt;
}
void SHMACTrafficTester::setSendDelay(sc_time delay)
{
	m_delaySend = delay;
}
void SHMACTrafficTester::setSendPacketCount(uint32_t cnt)
{
	m_sendPacketCount = cnt;
}

void SHMACTrafficTester::generateTraffic()
{
	while (m_sendPacketCount--)
	{
		outboundMemoryPort->receiveMessage(m_packetToSend);
		// TODO add fxns here to time the send calls
		wait(m_delaySend);
	}
}

void SHMACTrafficTester::receiveMessage(shmacsim::packet message)
{
	for (size_t i = 0; i < m_logEntries.size(); i++)
	{
		if (m_logEntries[i].packet == message)
		{
			m_logEntries[i].packetCount++;
			return;
		}
	}

	// entry not found, add new
	trafficLog newentry;
	newentry.packet = message;
	newentry.packetCount = 1;
	m_logEntries.push_back(newentry);
	m_receivedPacketCount++;
}

void SHMACTrafficTester::periodicReport()
{
	while (1)
	{
		wait(100, SC_NS);	// TODO make configurable!
		reportStats();
	}
}

void SHMACTrafficTester::reportStats()
{
	if (m_logEntries.size())
		std::cerr << "Traffic receiver with name " << this->name() << ":"
				<< std::endl;
	for (size_t i = 0; i < m_logEntries.size(); i++)
		std::cerr << "Received pkt with data " << std::hex
				<< m_logEntries[i].packet.data << std::dec << "\t"
				<< m_logEntries[i].packetCount << " times" << std::endl;
}
