/*
 * SHMACRouter.h
 *
 * model of the SHMAC router
 *
 *  Created on: 26 Feb 2013
 *      Author: Yaman Umuroglu
 */

#ifndef SHMACROUTER_H_
#define SHMACROUTER_H_

#include <systemc.h>
#include "shmac_interfaces.h"
#include "SHMACElement.h"
#include "SHMACMaster.h"
#include "SHMACSlave.h"

class SHMACRouter: public sc_module, public SHMACMemoryInterface
{
public:
	//SC_HAS_PROCESS(SHMACRouter);

	// port declarations
	// note the declaration SC_ZERO_OR_MORE_BOUND, since we allow some router
	// ports to be non-bound (i.e those on the edges of the grid)
	sc_port<SHMACMemoryInterface, 1, SC_ZERO_OR_MORE_BOUND> routerPorts[SHMAC_DIRECTION_COUNT];

	// channel and submodule pointer declarations

	// constructor
	SHMACRouter(sc_module_name instanceName, SHMACElement * parent = NULL);

	// process declarations: methods

	// process declarations: threads

	// helper function declarations

	// implemented from SHMACMemoryInterface
	void receiveMessage(shmacsim::packet message);

	SHMACElement* getParent() const
	{
		return m_parent;
	}

	virtual void setParent(SHMACElement* parent)
	{
		m_parent = parent;
	}

	// establish connection(s) between this router and given destination
	// derived router types should override and connect appropriately
	virtual void connectToRouter(SHMACRouter *dest, shmacsim::direction_enum directionTowardsDest);

	// return some data representing the abstraction level of router implementation
	// derived router types should override and change this - will be used to determine
	// if two routers are compatible (can be connected)
	virtual uint8_t getRouterModelType() {return 'H';};

	// establish two-way connection(s) between router and given master
	virtual void connectMaster(SHMACMaster * master);

	// establish two-way connection(s) between router and given slave
	virtual void connectSlave(SHMACSlave * slave);

	// establish two-way connection(s) between given from-to ports and given router dir
	virtual void connectSHMACMemInterface(shmacsim::direction_enum dir,
			sc_port<SHMACMemoryInterface> & fromRouter,
			sc_port<SHMACMemoryInterface> & toRouter);

protected:
	SHMACElement * m_parent;
	sc_mutex m_lock[SHMAC_DIRECTION_COUNT];

};

#endif /* SHMACROUTER_H_ */
