#ifndef SHMACROUTERCYCACC_H_
#define SHMACROUTERCYCACC_H_

#include <systemc.h>
#include "router/SHMACRouterArbiter.h"
#include "router/SHMACRouterCrossbar.h"
#include "router/SHMACRouterInputPort.h"
#include "router/SHMACRouterOutputPort.h"
#include "SHMACRouter.h"

#include "SHMACRTLToMemAdapter.h"
#include "SHMACMemToRTLAdapter.h"

class SHMACRouterCycAcc: public SHMACRouter
{
public:
	SC_HAS_PROCESS(SHMACRouterCycAcc);

	sc_in_clk clk;

	// data input signals
	sc_signal<bool> rx_req_signal[SHMAC_DIRECTION_COUNT];
	sc_signal<shmacsim::packet> rx_data_signal[SHMAC_DIRECTION_COUNT];
	sc_signal<bool> rx_ack_signal[SHMAC_DIRECTION_COUNT];

	// data inputs
	sc_in<bool> rx_req[SHMAC_DIRECTION_COUNT];
	sc_in<shmacsim::packet> rx_data[SHMAC_DIRECTION_COUNT];
	sc_out<bool> rx_ack[SHMAC_DIRECTION_COUNT];

	// data outputs
	sc_out<bool> tx_req[SHMAC_DIRECTION_COUNT];
	sc_out<shmacsim::packet> tx_data[SHMAC_DIRECTION_COUNT];
	sc_in<bool> tx_ack[SHMAC_DIRECTION_COUNT];

	virtual void setParent(SHMACElement* parent);

	SHMACRouterCycAcc(sc_module_name name, SHMACElement * parent);
	~SHMACRouterCycAcc();

	void plugUnboundTxPorts();

	friend void sc_trace(sc_trace_file* tf, const SHMACRouterCycAcc& val,
			const std::string& nm);

	// router model type: 'R' (for RTL)
	virtual uint8_t getRouterModelType() {return 'R';};

	// establish connection(s) between this router and given destination
	// derived router types should override and connect appropriately
	virtual void connectToRouter(SHMACRouter *dest,	shmacsim::direction_enum directionTowardsDest);

	// SystemC hook - called at the end of the elaboration phase, after the HW is constructed
	// we use this opportunity to call plugUnboundTxCalls
    virtual void before_end_of_elaboration();

	// establish two-way connection(s) between given from-to ports and given router dir
	virtual void connectSHMACMemInterface(shmacsim::direction_enum dir,
			sc_port<SHMACMemoryInterface> & fromRouter,
			sc_port<SHMACMemoryInterface> & toRouter);

	SHMACRouterArbiter * m_arbiters[SHMAC_DIRECTION_COUNT];

protected:
	// router internal elements
	SHMACRouterCrossbar * m_switch;
	SHMACRouterInputPort * m_inputPorts[SHMAC_DIRECTION_COUNT];
	SHMACRouterOutputPort * m_outputPorts[SHMAC_DIRECTION_COUNT];


	// signal type adapters
	SHMACRTLToMemAdapter * m_adapterOut[SHMAC_DIRECTION_COUNT];
	SHMACMemToRTLAdapter * m_adapterIn[SHMAC_DIRECTION_COUNT];

	// signals for router internal element connections
	sc_signal<bool> m_arbiter_enable[SHMAC_DIRECTION_COUNT]; // output port -> arbiter
	sc_signal<bool> m_grant_valid[SHMAC_DIRECTION_COUNT]; // arbiter -> output port
	sc_signal<uint8_t> m_grant[SHMAC_DIRECTION_COUNT]; // arbiter -> output port
	sc_signal<bool> m_input_router_ack[SHMAC_DIRECTION_COUNT]; // switch -> input port
	sc_signal<shmacsim::packet> m_input_router_packet[SHMAC_DIRECTION_COUNT]; // input port -> switch
	sc_signal<bool> m_router_output_ack[SHMAC_DIRECTION_COUNT]; // output port -> switch
	sc_signal<shmacsim::packet> m_router_output_data[SHMAC_DIRECTION_COUNT]; // switch -> output port
	sc_signal<bool> m_req[SHMAC_DIRECTION_COUNT][SHMAC_DIRECTION_COUNT]; // input port -> arbiter and output port, not 1-1
	sc_signal<bool> m_setsw[SHMAC_DIRECTION_COUNT][SHMAC_DIRECTION_COUNT]; // output port -> switch

};

#endif /* SHMACROUTERCYCACC_H_ */
