#ifndef SHMACROUTEROUTPUTPORT_H_
#define SHMACROUTEROUTPUTPORT_H_

#include <systemc.h>
#include "shmac_simutils.h"
#include "shmac_types.h"

class SHMACRouterOutputPort : public sc_module
{
public:
	SC_HAS_PROCESS(SHMACRouterOutputPort);

	sc_in_clk clk;

	// Control
	sc_out<bool> arbiter_enable;
	sc_out<bool> set_switch[SHMAC_DIRECTION_COUNT];
	sc_in<uint8_t> grant;
	sc_in<bool> grant_valid;

	// Incoming data
	sc_in<bool> router_output_req[SHMAC_DIRECTION_COUNT];
	sc_out<bool> router_output_ack;
	sc_in<shmacsim::packet> router_output_data;

	// Outgoing data
	sc_out<bool> tx_req;
	sc_out<shmacsim::packet> tx_data;
	sc_in<bool> tx_ack;

	SHMACRouterOutputPort(sc_module_name name);

	void sync_proc();
	void fsm();

protected:
	typedef enum {
		st_idle,
		st_switch,
		st_switch_ack,
		st_switch_wait,
		st_send,
		st_handshake
	} state_type;

	sc_signal<state_type> state, next_state;

	// buffered signals
	sc_signal<bool> router_output_ack_i;
	sc_signal<bool> tx_req_i;
	sc_signal<shmacsim::packet> tx_data_r;
	sc_signal<shmacsim::packet> tx_data_i;
	sc_signal<bool> arbiter_enable_i;
	sc_signal<bool> set_switch_i[SHMAC_DIRECTION_COUNT];
	sc_signal<uint8_t> grant_r, grant_i;

};

#endif /* SHMACROUTEROUTPUTPORT_H_ */
