/*
 * SHMACDualPortRAM.cpp
 *
 *  Created on: Jul 3, 2013
 *      Author: maltanar
 */

#include "SHMACDualPortRAM.h"
#include "SHMACPacketTracker.h"
#include "shmac_typeutils.h"

SHMACDualPortRAM::SHMACDualPortRAM(sc_module_name name, uint32_t memsize_bytes, SHMACElement * parent, bool readOnly) :
	SHMACSlave(name, memsize_bytes, parent, readOnly)
{
	// most initialization handled by constructor call to SHMACSlave
	// construct SHMACSlavePort object to handle the second mem port here
	m_receivedMessage_port2 = dummyPacket;
	m_secondaryPort = new SHMACSlavePort();
	m_secondaryPort->attachToSlave(this, 1);
	secondaryInboundPort.bind(*m_secondaryPort);

	SC_THREAD(performMemoryAccesses_port2);
}

SHMACDualPortRAM::~SHMACDualPortRAM()
{
	delete m_secondaryPort;
}

void SHMACDualPortRAM::performMemoryAccesses_port2()
{
	while (1)
	{
		// wait until request received
		wait(m_pendingRequest_port2);
		// handle tracker, if exists
		if (m_receivedMessage_port2.tag)
		{
			((SHMACPacketTracker *) m_receivedMessage_port2.tag)->registerLocation(
					"memory");
			// end packet tracking for write pkt
			if (m_receivedMessage_port2.write_enable == shmacsim::PACKET_WREN_WRITE4)
			{
				delete ((SHMACPacketTracker *) m_receivedMessage_port2.tag);
				m_receivedMessage_port2.tag = NULL;
			}
			// other packets tracers will continue on to the reply
		}
		handleMessage_port2(m_receivedMessage_port2);
		m_requestServed_port2.notify();
	}
}

// execution context: this SHMACSlave's own SC_THREAD (performMemoryAccesses_port2)
void SHMACDualPortRAM::handleMessage_port2(shmacsim::packet message)
{
	// call the internal handler to carry out the operations
	shmacsim::packet response = memoryAccess(message);

	// send reply only for read operations, no reply needed for stores
	// note that SHMACSlave doesn't care about the sync flag, unlike SHMACLockSlave
	if (message.write_enable == shmacsim::PACKET_WREN_READ4)
		secondaryOutboundPort->receiveMessage(response);
}
