/*
 * SHMACJumpSlave.cpp
 *
 *  Created on: Jun 18, 2013
 *      Author: maltanar
 */

#include "SHMACJumpSlave.h"
#include "shmac_simutils.h"

SHMACJumpSlave::SHMACJumpSlave(sc_module_name name,
		SHMACElement * parent) :
		SHMACSlave(name, 16, parent, true)
{
	// no need for extra initialization,
	// since this slave dynamically responds to all requests
}

SHMACJumpSlave::~SHMACJumpSlave()
{

}

void SHMACJumpSlave::memorySetWord(SHMAC_ADDR address, SHMAC_WORD data)
{
	sc_assert(0);	// jump slave is read-only
}

SHMAC_WORD SHMACJumpSlave::memoryGetWord(SHMAC_ADDR address)
{
	shmacsim::tile_coord requesterCoord = m_receivedMessage.sender;
	SHMAC_WORD ret_val = 0;

	//std::cerr << "Jump slave got request from " << requesterCoord << " addr " << address << std::endl;

	switch(address)
	{
		case SHMAC_JUMPSLAVE_ADDR_LUI:
			//  lui         0     rt       i     j       0
		    // "001111"&"00000"&"01100"&"iiii"&"jjjj"&"00000000"
			ret_val = 0x3C0C0000 | ((requesterCoord.i & 0x0F) << 12) | ((requesterCoord.j & 0x0F) << 8);
		break;
		case SHMAC_JUMPSLAVE_ADDR_ORI:
			//    ori       rs     rt      imm
			// "001101"&"01100"&"01100"&"0000000000000000"
			// OR register 12 with itself
			ret_val = 0x358C0000;
		break;
		case SHMAC_JUMPSLAVE_ADDR_JR:
			//   special   rs          0       hint     jr
			// "000000"&"01100"&"0000000000"&"00000"&"001000"
			ret_val = 0x01800008;
		break;
		case SHMAC_JUMPSLAVE_ADDR_NOP:
			// nop, all zeroes
			ret_val = 0x0;
		break;
		default:
		sc_assert(0);	// cannot be handled by jump slave
		break;
	}

	//std::cerr << "jump slave returning " << std::hex << ret_val << std::dec << std::endl;

	// need endian adjustment since we are crossing the host-target border
	ret_val = shmacsim::convertHostToTargetEndian_32bit(ret_val);

	return ret_val;
}
