/*
 * SHMACLockSlave.h
 *
 *  Created on: 19 Mar 2013
 *      Author: Yaman Umuroglu
 */

#ifndef SHMACLOCKSLAVE_H_
#define SHMACLOCKSLAVE_H_

#include <SHMACSlave.h>
#include "shmac_types.h"

class SHMACLockSlave: public SHMACSlave
{
public:
	SHMACLockSlave(sc_module_name name, uint32_t memsize_bytes,
			SHMACElement * parent = NULL);
	~SHMACLockSlave();

protected:
	// structure representing one entry of the LL/SC bookkeeping table
	typedef struct
	{
		shmacsim::bus32 address;
		bool valid;
	} lockTableEntry;

	lockTableEntry m_lockTable[SHMAC_MAX_GRIDDIM * SHMAC_MAX_GRIDDIM];

	// structure for keeping LL∕SC statistics
	typedef struct
	{
		uint32_t count_ll;
		uint32_t count_sc_ok;
		uint32_t count_sc_fail;
	} lockStatEntry;

	// table for keeping statistics - how many times a SC has failed from a processor
	lockStatEntry m_lockStats[SHMAC_MAX_GRIDDIM * SHMAC_MAX_GRIDDIM];

	// invalidate entries with matching address in table
	void invalidateLockEntriesForAddress(shmacsim::bus32 addr);

	virtual void handleMessage(shmacsim::packet message);

};

#endif /* SHMACLOCKSLAVE_H_ */
