/*
 * SHMACSlave.h
 *
 * models the SHMAC slave elements, which correspond to memory of some sort
 *
 *  Created on: 26 Feb 2013
 *      Author: Yaman Umuroglu
 */

#ifndef SHMACSLAVE_H_
#define SHMACSLAVE_H_

#include <systemc.h>
#include "shmac_interfaces.h"
#include "SHMACElement.h"

class SHMACSlave: public sc_module, public SHMACMemoryInterface
{
public:
	SC_HAS_PROCESS(SHMACSlave);

	sc_port<SHMACMemoryInterface> outboundMemoryPort;
	sc_port<SHMACMemoryInterface> inboundMemoryPort;

	SHMACSlave(sc_module_name name, uint32_t memsize_bytes, SHMACElement * parent = NULL, bool readOnly = false);
	virtual ~SHMACSlave();

	void setDelayEnabled(bool delayEnabled);

	virtual void receiveMessage(shmacsim::packet message);

	void exportContents(uint8_t * buffer, uint32_t bufferSize);
	void exportContents(const char * fileName);
	bool importContents(uint8_t * buffer, uint32_t bufferSize);
	bool importContents(const char * fileName, uint32_t * importedSize = NULL);

	SHMACElement* getParent() const
	{
		return m_parent;
	}

	void setParent(SHMACElement* parent)
	{
		m_parent = parent;
	}

	// return the allocated memory size of slave
	uint32_t getSize()
	{
		return m_allocSize;
	}

	// the set of accessors functions below is a part of the half-complete
	// SHMACSlavePort refactoring work, they are used solely by SHMACSlavePort
	// at the moment and don't make all that much sense
	virtual sc_mutex * getMemMutex(int portNumber) {return &m_memLock;};
	virtual sc_event * getRequestEvent(int portNumber) {return &m_pendingRequest;};
	virtual sc_event * getServedEvent(int portNumber) {return &m_requestServed;};
	virtual shmacsim::packet * getPacketBuffer(int portNumber) {return &m_receivedMessage;};
	virtual uint32_t getPortCount() {return 1;};

protected:
	SHMACElement * m_parent;
	uint8_t * m_memory;
	uint32_t m_allocSize;
	sc_mutex m_memLock;
	bool m_readOnly;

	virtual void performMemoryAccesses();
	virtual void handleMessage(shmacsim::packet message);

	// the core memory access function, calling the other access functions
	virtual shmacsim::packet memoryAccess(shmacsim::packet message);
	shmacsim::packet m_receivedMessage;
	sc_event m_pendingRequest, m_requestServed;

	// virtual methods for memory access - reimplement as needed for derivatives
	virtual void memorySetWord(SHMAC_ADDR address, SHMAC_WORD data);
	virtual SHMAC_WORD memoryGetWord(SHMAC_ADDR address);

	// TODO eventually rename this to sendMessage, after SHMACMemIF func gets renamed
	virtual void sendMessage(shmacsim::packet message);
};

#endif /* SHMACSLAVE_H_ */
