/*
 * SHMACSlavePort.cpp
 *
 *  Created on: Jul 3, 2013
 *      Author: maltanar
 */

#include "SHMACSlavePort.h"
#include "shmac_simutils.h"

SHMACSlavePort::SHMACSlavePort()
{
	m_memLock = NULL;
	m_pendingRequest = m_requestServed = NULL;
	m_packetBuffer = NULL;
	m_attached = false;
}

SHMACSlavePort::~SHMACSlavePort()
{

}

void SHMACSlavePort::attachToSlave(SHMACSlave * slave, int portIndex)
{
	m_memLock = slave->getMemMutex(portIndex);
	m_pendingRequest = slave->getRequestEvent(portIndex);
	m_requestServed = slave->getServedEvent(portIndex);
	m_packetBuffer = slave->getPacketBuffer(portIndex);

	m_attached = true;
}

void SHMACSlavePort::receiveMessage(shmacsim::packet message)
{
	// ensure this port is attached to a slave unit
	sc_assert(m_attached);
	m_memLock->lock();
	// 1 cc delay before reading in request
	// memory state machine: idle->ram_wait
	shmacsim::waitClockCycles(1);
	*m_packetBuffer = message;
	m_pendingRequest->notify();
	wait(*m_requestServed);
	// memory state machine, waiting for clock edges
	shmacsim::waitClockCycles(1);
	m_memLock->unlock();
}
