// unit test for SHMACRouterArbiter

#ifndef UNITTEST_SHMACROUTERARBITER_H_
#define UNITTEST_SHMACROUTERARBITER_H_

#include "router/SHMACRouterArbiter.h"

class UnitTest_SHMACRouterArbiter: public sc_module
{
public:
	SC_HAS_PROCESS(UnitTest_SHMACRouterArbiter);

	sc_signal<bool> enable;
	sc_signal<bool> req[SHMAC_DIRECTION_COUNT];
	sc_signal<uint8_t> grant;
	sc_signal<bool> valid;
	sc_clock * clk;
	SHMACRouterArbiter * uut;
	sc_trace_file* Tf;

	static bool executeUnitTest()
	{
		UnitTest_SHMACRouterArbiter * unitTest =
				new UnitTest_SHMACRouterArbiter("UnitTest_SHMACRouterArbiter");
		// TODO unit test: configure sim duration?
		sc_start(400, SC_NS);

		delete unitTest;
		// TODO unit test: check and report test result
		return true;
	}

	// constructor
	UnitTest_SHMACRouterArbiter(sc_module_name instanceName) :
			sc_module(instanceName)
	{
		// TODO unit test: make clk parameters configurable for unit test?
		clk = new sc_clock("clk", 2, SC_NS, 0.5);
		SC_THREAD(run);

		// instantiate the unit to test and bind its ports to local signals
		uut = new SHMACRouterArbiter("arb");
		uut->enable(enable);
		uut->clk(*clk);
		uut->grant_o(grant);
		uut->valid(valid);

		// set up the trace file
		Tf = sc_create_vcd_trace_file("UnitTest_SHMACRouterArbiter");
		((vcd_trace_file*) Tf)->set_time_unit(1, SC_NS);

		// add the signals to be traced
		sc_trace(Tf, enable, "enable");
		sc_trace(Tf, *clk, "clk");
		sc_trace(Tf, grant, "grant");
		sc_trace(Tf, valid, "valid");

		char nb[15];
		SHMAC_FOREACH_DIR(dir)
		{
			uut->req_i[dir](req[dir]);
			sprintf(nb, "req_%d", dir);
			sc_trace(Tf, req[dir], nb);
		}
	}

	~UnitTest_SHMACRouterArbiter()
	{
		sc_close_vcd_trace_file(Tf);
		delete uut;
		delete clk;
	}

	void waitClockPosedge(int n = 1)
	{
		while (n--)
			wait(clk->posedge_event());
	}

	void run()
	{
		// TODO unit test: needs assertions / output on test results!
		SHMAC_FOREACH_DIR(dir)
			req[dir].write(false);
		enable.write(true);

		waitClockPosedge();

		SHMAC_FOREACH_DIR(dir)
			req[dir].write(true);

		int n = 10;
		while (n--)
		{
			enable.write(true);
			waitClockPosedge();
			enable.write(false);
			waitClockPosedge(4);
		}

		SHMAC_FOREACH_DIR(dir)
			req[dir].write(dir % 2 == 0 ? 0 : 1);
		// req.write("101010");

		n = 4;
		while (n--)
		{
			enable.write(true);
			waitClockPosedge();
			enable.write(false);
			waitClockPosedge(4);
		}

		//req.write("010101");
		SHMAC_FOREACH_DIR(dir)
			req[dir].write(dir % 2 != 0 ? 0 : 1);
		n = 4;
		while (n--)
		{
			enable.write(true);
			waitClockPosedge();
			enable.write(false);
			waitClockPosedge(4);
		}

		SHMAC_FOREACH_DIR(dir)
			req[dir].write(false);
		enable.write(true);
	}
};

#endif /* UNITTEST_SHMACROUTERARBITER_H_ */
