#ifndef UNITTEST_SHMACROUTERCYCACC_H_
#define UNITTEST_SHMACROUTERCYCACC_H_

#include "router/SHMACRouterCycAcc.h"
#include "shmac_typeutils.h"
#include "SHMACElement.h"

// TODO rewrite to use SHMACMemoryInterface when done

class UnitTest_SHMACRouterCycAcc: public sc_module
{
public:
	SC_HAS_PROCESS(UnitTest_SHMACRouterCycAcc);

	sc_time clkprd;

	sc_clock * clk;
	SHMACElement * t1, *t2, *t3;
	SHMACRouterCycAcc * uut[3];
	sc_trace_file* Tf;

	sc_signal<bool> one_west_tx_ack;
	sc_signal<bool> two_south_tx_ack;
	sc_signal<bool> three_mem_tx_ack;
	sc_signal<bool> three_east_tx_ack;

	static bool executeUnitTest()
	{
		UnitTest_SHMACRouterCycAcc * unitTest = new UnitTest_SHMACRouterCycAcc(
				"UnitTest_SHMACRouterCycAcc");
		// TODO unit test: configure sim duration?

		sc_start(5000, SC_NS);

		delete unitTest;
		// TODO unit test: check and report test result
		return true;
	}

	// constructor
	UnitTest_SHMACRouterCycAcc(sc_module_name instanceName) :
			sc_module(instanceName)
	{
		SC_THREAD(run_1);
		SC_THREAD(run_2);
		SC_THREAD(run_3);
		SC_THREAD(run_4);

		// TODO unit test: make clk parameters configurable for unit test?
		clk = new sc_clock("clk", 10, SC_NS, 0.5);
		clkprd = sc_time(10, SC_NS);

		// instantiate the unit to test and bind its ports to local signals
		t1 = new SHMACElement(1, 1);
		uut[0] = new SHMACRouterCycAcc("one", t1);
		uut[0]->tx_ack[shmacsim::DIR_WEST].bind(one_west_tx_ack);

		t2 = new SHMACElement(1, 2);
		uut[1] = new SHMACRouterCycAcc("two", t2);
		uut[1]->tx_ack[shmacsim::DIR_SOUTH].bind(two_south_tx_ack);

		t3 = new SHMACElement(1, 3);
		uut[2] = new SHMACRouterCycAcc("three", t3);
		uut[2]->tx_ack[shmacsim::DIR_INTERN_MEM].bind(three_mem_tx_ack);
		uut[2]->tx_ack[shmacsim::DIR_EAST].bind(three_east_tx_ack);

		uut[0]->clk(*clk);
		uut[1]->clk(*clk);
		uut[2]->clk(*clk);

		/*
		 -----           (0,1)        (0,2)          (0,3)
		 ---               |            |              |
		 --    (1,0) - [one(1,1)] - [two(1,2)] - [three(1,3)] - (1,4)
		 ---               |            |              |
		 -----           (2,1)        (2,2)          (2,3)
		 */

		uut[0]->connectToRouter(uut[1], shmacsim::DIR_EAST);
		uut[1]->connectToRouter(uut[0], shmacsim::DIR_WEST);

		uut[1]->connectToRouter(uut[2], shmacsim::DIR_EAST);
		uut[2]->connectToRouter(uut[1], shmacsim::DIR_WEST);

		uut[0]->plugUnboundTxPorts();
		uut[1]->plugUnboundTxPorts();
		uut[2]->plugUnboundTxPorts();

		//std::cout << "bind count: " << uut[0]->tx_req[shmacsim::DIR_WEST].bind_count() << std::endl;

		// set up the trace file
		Tf = sc_create_vcd_trace_file("UnitTest_SHMACRouterCycAcc");
		((vcd_trace_file*) Tf)->set_time_unit(1, SC_NS);

		// add the signals to be traced
		char nb[30];
		for (int x = 0; x < 3; x++)
		{
			sprintf(nb, "r%d", x + 1);
			sc_trace(Tf, *uut[x], nb);

			/*SHMAC_FOREACH_DIR(dir)
			 {

			 sprintf(nb, "%d_rx_req_%s", x + 1,
			 getDirectionName(dir).c_str());
			 sc_trace(Tf, uut[x]->rx_req[dir], nb);

			 sprintf(nb, "%d_rx_data_%s", x + 1,
			 getDirectionName(dir).c_str());
			 sc_trace(Tf, uut[x]->rx_data[dir], nb);
			 }*/
		}

		sc_trace(Tf, *clk, "clk");

	}

	~UnitTest_SHMACRouterCycAcc()
	{
		sc_close_vcd_trace_file(Tf);
		delete uut[0];
		delete uut[1];
		delete uut[2];
		delete t1;
		delete t2;
		delete t3;
		delete clk;
	}

	void waitClockPosedge(int n = 1)
	{
		while (n)
		{
			n--;
			wait(clk->posedge_event());
		}
	}

	void DebugPrint(int testid, int pktid, string msg)
	{
		std::cout << "Test " << testid << " pkt# " << pktid << " : " << msg << " at cc# " << sc_time_stamp() / clkprd << std::endl;
	}

	void run_1() // [2,1] --> [1,3]
	{
		shmacsim::packet testPacket;
		testPacket.address = 0x22000000;
		testPacket.data = 0xbeefdead;
		testPacket.req = true;
		testPacket.flag = shmacsim::PACKET_FLAG_NONE;
		testPacket.sender =
		{	2, 1};
		testPacket.write_enable = shmacsim::PACKET_WREN_READ4;

		for (int i = 0; i < 10; i++)
		{
			wait(200, SC_NS);

			uut[0]->rx_data_signal[shmacsim::DIR_SOUTH] = testPacket;
			uut[0]->rx_req_signal[shmacsim::DIR_SOUTH] = true;

			wait(uut[0]->rx_ack_signal[shmacsim::DIR_SOUTH].posedge_event());
			DebugPrint(1, i, "RX ACK");
			waitClockPosedge(2);

			uut[0]->rx_req_signal[shmacsim::DIR_SOUTH] = false;

			wait(uut[1]->tx_req[shmacsim::DIR_SOUTH].posedge_event());
			DebugPrint(1, i, "TX REQ");
			waitClockPosedge(1);

			two_south_tx_ack = true;
			wait(uut[1]->tx_req[shmacsim::DIR_SOUTH].negedge_event());
			two_south_tx_ack = false;
		}

		std::cout << "Test 1: (2,1)->(1,1)->(1,2)->(1,3) succeeded "
				<< std::endl;
	}

	void run_2() // [0,1] ---> [2,2]
	{
		shmacsim::packet testPacket;
		testPacket.address = 0x13000000;
		testPacket.data = 0xdeadbeef;
		testPacket.req = true;
		testPacket.flag = shmacsim::PACKET_FLAG_NONE;
		testPacket.sender =
		{	0, 1};
		testPacket.write_enable = shmacsim::PACKET_WREN_READ4;

		for (int i = 0; i < 10; i++)
		{
			waitClockPosedge(2);

			uut[0]->rx_data_signal[shmacsim::DIR_NORTH] = testPacket;
			uut[0]->rx_req_signal[shmacsim::DIR_NORTH] = true;

			wait(uut[0]->rx_ack_signal[shmacsim::DIR_NORTH].posedge_event());
			DebugPrint(2, i, "RX ACK");
			waitClockPosedge(2);

			uut[0]->rx_req_signal[shmacsim::DIR_NORTH] = false;

			wait(uut[2]->tx_req[shmacsim::DIR_INTERN_MEM].posedge_event());
			DebugPrint(2, i, "TX REQ");
			waitClockPosedge(1);

			three_mem_tx_ack = true;

			wait(uut[2]->tx_req[shmacsim::DIR_INTERN_MEM].negedge_event());
			three_mem_tx_ack = false;
		}

		std::cout << "Test 2: (0,1)->(1,1)->(1,2)->(2,2) succeeded "
				<< std::endl;
	}

	void run_3() // [1,0] ---> [1,7]
	{
		shmacsim::packet testPacket;
		testPacket.address = 0x17000000;
		testPacket.data = 0xfeedbead;
		testPacket.req = true;
		testPacket.flag = shmacsim::PACKET_FLAG_NONE;
		testPacket.sender =
		{	1, 0};
		testPacket.write_enable = shmacsim::PACKET_WREN_READ4;

		for (int i = 0; i < 10; i++)
		{
			waitClockPosedge(2);

			uut[0]->rx_data_signal[shmacsim::DIR_WEST] = testPacket;
			uut[0]->rx_req_signal[shmacsim::DIR_WEST] = true;

			wait(uut[0]->rx_ack_signal[shmacsim::DIR_WEST].posedge_event());
			DebugPrint(3, i, "RX ACK");
			waitClockPosedge(2);

			uut[0]->rx_req_signal[shmacsim::DIR_WEST] = false;

			wait(uut[2]->tx_req[shmacsim::DIR_EAST].posedge_event());
			DebugPrint(3, i, "TX REQ");
			waitClockPosedge(1);

			three_east_tx_ack = true;

			wait(uut[2]->tx_req[shmacsim::DIR_EAST].negedge_event());
			three_east_tx_ack = false;
		}

		std::cout << "Test 3: (1,0)->(1,1)->(1,2)->(1,3)->E succeeded "
				<< std::endl;
	}

	void run_4() // [0,3] ---> [1,0]
	{
		shmacsim::packet testPacket;
		testPacket.address = 0x10000000;
		testPacket.data = 0xbeadfeed;
		testPacket.req = true;
		testPacket.flag = shmacsim::PACKET_FLAG_NONE;
		testPacket.sender =
		{	0, 3};
		testPacket.write_enable = shmacsim::PACKET_WREN_READ4;

		for (int i = 0; i < 10; i++)
		{
			wait(100, SC_NS);

			uut[2]->rx_data_signal[shmacsim::DIR_NORTH] = testPacket;
			uut[2]->rx_req_signal[shmacsim::DIR_NORTH] = true;

			wait(uut[2]->rx_ack_signal[shmacsim::DIR_NORTH].posedge_event());
			DebugPrint(4, i, "RX ACK");

			waitClockPosedge(2);

			uut[2]->rx_req_signal[shmacsim::DIR_NORTH] = false;

			wait(uut[0]->tx_req[shmacsim::DIR_WEST].posedge_event());
			DebugPrint(4, i, "TX REQ");
			waitClockPosedge(1);

			one_west_tx_ack = true;

			wait(uut[0]->tx_req[shmacsim::DIR_WEST].negedge_event());
			one_west_tx_ack = false;
		}

		std::cout << "Test 4: (0,3)->(1,3)->(1,2)->(1,1)->W succeeded "
				<< std::endl;
	}
};

#endif /* UNITTEST_SHMACROUTERCYCACC_H_ */
