#ifndef UNITTEST_SHMACROUTERINPUTPORT_H_
#define UNITTEST_SHMACROUTERINPUTPORT_H_

#include "router/SHMACRouterInputPort.h"

class UnitTest_SHMACRouterInputPort: public sc_module
{
public:
	SC_HAS_PROCESS(UnitTest_SHMACRouterInputPort);

	sc_signal<bool> rx_req;
	sc_signal<shmacsim::packet> rx_data;
	sc_signal<bool> rx_ack;

	sc_signal<bool> input_router_ack;
	sc_signal<bool> input_router_req[SHMAC_DIRECTION_COUNT];
	sc_signal<shmacsim::packet> input_router_packet;

	sc_clock * clk;
	SHMACRouterInputPort * uut;
	sc_trace_file* Tf;

	static bool executeUnitTest()
	{
		UnitTest_SHMACRouterInputPort * unitTest =
				new UnitTest_SHMACRouterInputPort(
						"UnitTest_SHMACRouterInputPort");
		// TODO unit test: configure sim duration?
		sc_start(400, SC_NS);

		delete unitTest;
		// TODO unit test: check and report test result
		return true;
	}

	// constructor
	UnitTest_SHMACRouterInputPort(sc_module_name instanceName) :
			sc_module(instanceName)
	{
		// TODO unit test: make clk parameters configurable for unit test?
		clk = new sc_clock("clk", 2, SC_NS, 0.5);
		SC_THREAD(run);

		// instantiate the unit to test and bind its ports to local signals
		uut = new SHMACRouterInputPort("ip", { 1, 1 });
		uut->input_router_ack(input_router_ack);
		uut->input_router_packet(input_router_packet);
		uut->rx_ack(rx_ack);
		uut->rx_data(rx_data);
		uut->rx_req(rx_req);
		uut->clk(*clk);

		// set up the trace file
		Tf = sc_create_vcd_trace_file("UnitTest_SHMACRouterInputPort");
		((vcd_trace_file*) Tf)->set_time_unit(1, SC_NS);

		// add the signals to be traced
		sc_trace(Tf, input_router_ack, "input_router_ack");
		sc_trace(Tf, input_router_packet, "input_router_packet");
		sc_trace(Tf, rx_ack, "rx_ack");
		sc_trace(Tf, rx_data, "rx_data");
		sc_trace(Tf, rx_req, "rx_req");
		sc_trace(Tf, *clk, "clk");

		char nb[15];
		SHMAC_FOREACH_DIR(dir)
		{
			sprintf(nb, "req_%d", dir);
			uut->input_router_req[dir](input_router_req[dir]);
			sc_trace(Tf, input_router_req[dir], nb);
		}
	}

	~UnitTest_SHMACRouterInputPort()
	{
		sc_close_vcd_trace_file(Tf);
		delete uut;
		delete clk;
	}

	void waitClockPosedge(int n = 1)
	{
		while (n)
		{
			n--;
			wait(clk->posedge_event());
		}
	}

	void run()
	{
		// construct a test packet to be transported
		shmacsim::packet testPacket;
		testPacket.data = 0xcafebabe;
		testPacket.req = true;
		testPacket.sender =
		{	1,1};
		testPacket.write_enable = shmacsim::PACKET_WREN_WRITE4;
		testPacket.flag = shmacsim::PACKET_FLAG_NONE;

		waitClockPosedge();

		shmacsim::direction_enum goldenRoutes[] = { shmacsim::DIR_WEST,
				shmacsim::DIR_NORTH, shmacsim::DIR_EAST, shmacsim::DIR_WEST,
				shmacsim::DIR_INTERN_MEM, shmacsim::DIR_EAST,
				shmacsim::DIR_WEST, shmacsim::DIR_SOUTH, shmacsim::DIR_EAST };

		uint8_t cnt = 0;

		for (int i = -1; i < 2; i++)
			for (int j = -1; j < 2; j++)
			{
				testPacket.address = setAddressTileCoord(0,
						{ (shmacsim::coord_int) (1 + i),
								(shmacsim::coord_int) (1 + j) });
				input_router_ack = false;
				rx_data = testPacket;
				rx_req = true;
				wait(rx_ack.posedge_event());
				rx_req = false;
				wait(rx_ack.negedge_event());
				// assert correct routing destination
				SHMAC_FOREACH_DIR(dir)
					sc_assert(input_router_req[dir].read() == (dir == goldenRoutes[cnt] ? true : false));
				cnt++;
				input_router_ack = true;
				// assert that the packet passed thru the input port unmodified
				sc_assert(input_router_packet.read() == testPacket);
				//wait(input_router_req.value_changed_event());
				waitClockPosedge();
			}

		/*testPacket.address = setAddressTileCoord(0, {0,0});
		 input_router_ack = false;
		 rx_data = testPacket;
		 rx_req = true;
		 wait(rx_ack.posedge_event());
		 rx_req = false;
		 wait(rx_ack.negedge_event());
		 // & input_router_req.value_changed_event());
		 // TODO assert correct input_router_req
		 input_router_ack = true;
		 // assert that the packet passed thru the input port unmodified
		 sc_assert(input_router_packet.read() == testPacket);
		 wait(input_router_req.value_changed_event());

		 testPacket.address = setAddressTileCoord(0, {2,2});
		 input_router_ack = false;
		 rx_data = testPacket;
		 rx_req = true;
		 wait(rx_ack.posedge_event());
		 rx_req = false;
		 wait(rx_ack.negedge_event());
		 // & input_router_req.value_changed_event());
		 // TODO assert correct input_router_req
		 input_router_ack = true;
		 // assert that the packet passed thru the input port unmodified
		 sc_assert(input_router_packet.read() == testPacket);
		 wait(input_router_req.value_changed_event());
		 input_router_ack = false;*/

		std::cout << "Unit test SHMACRouterInputPort passed!" << std::endl;
	}
};

#endif /* UNITTEST_SHMACROUTERINPUTPORT_H_ */
