#ifndef UNITTEST_SHMACROUTEROUTPUTPORT_H_
#define UNITTEST_SHMACROUTEROUTPUTPORT_H_

#include "router/SHMACRouterOutputPort.h"

class UnitTest_SHMACRouterOutputPort: public sc_module
{
public:
	SC_HAS_PROCESS(UnitTest_SHMACRouterOutputPort);

	// Control
	sc_signal<bool> arbiter_enable;
	sc_signal<bool> set_switch[SHMAC_DIRECTION_COUNT];
	sc_signal<uint8_t> grant;
	sc_signal<bool> grant_valid;

	// Incoming data
	sc_signal<bool> router_output_req[SHMAC_DIRECTION_COUNT];
	sc_signal<bool> router_output_ack;
	sc_signal<shmacsim::packet> router_output_data;

	// Outgoing data
	sc_signal<bool> tx_req;
	sc_signal<shmacsim::packet> tx_data;
	sc_signal<bool> tx_ack;

	sc_clock * clk;
	SHMACRouterOutputPort * uut;
	sc_trace_file* Tf;

	static bool executeUnitTest()
	{
		UnitTest_SHMACRouterOutputPort * unitTest =
				new UnitTest_SHMACRouterOutputPort(
						"UnitTest_SHMACRouterOutputPort");
		// TODO unit test: configure sim duration?
		sc_start(400, SC_NS);

		delete unitTest;
		// TODO unit test: check and report test result
		return true;
	}

	// constructor
	UnitTest_SHMACRouterOutputPort(sc_module_name instanceName) :
			sc_module(instanceName)
	{
		// TODO unit test: make clk parameters configurable for unit test?
		clk = new sc_clock("clk", 2, SC_NS, 0.5);
		SC_THREAD(run);

		// instantiate the unit to test and bind its ports to local signals
		uut = new SHMACRouterOutputPort("uut");

		uut->arbiter_enable(arbiter_enable);
		uut->grant(grant);
		uut->grant_valid(grant_valid);
		uut->router_output_ack(router_output_ack);
		uut->router_output_data(router_output_data);
		uut->tx_req(tx_req);
		uut->tx_data(tx_data);
		uut->tx_ack(tx_ack);

		uut->clk(*clk);

		// set up the trace file
		Tf = sc_create_vcd_trace_file("UnitTest_SHMACRouterOutputPort");
		((vcd_trace_file*) Tf)->set_time_unit(1, SC_NS);

		// add the signals to be traced
		sc_trace(Tf, arbiter_enable, "arbiter_enable");
		sc_trace(Tf, grant, "grant");
		sc_trace(Tf, grant_valid, "grant_valid");
		sc_trace(Tf, router_output_ack, "router_output_ack");
		sc_trace(Tf, router_output_data, "router_output_data");
		sc_trace(Tf, tx_req, "tx_req");
		sc_trace(Tf, tx_data, "tx_data");
		sc_trace(Tf, tx_ack, "tx_ack");

		sc_trace(Tf, *clk, "clk");

		char nb[15];
		SHMAC_FOREACH_DIR(dir)
		{
			uut->set_switch[dir](set_switch[dir]);
			sprintf(nb, "set_sw_%d", dir);
			sc_trace(Tf, set_switch[dir], nb);
			uut->router_output_req[dir](router_output_req[dir]);
			sprintf(nb, "req_%d", dir);
			sc_trace(Tf, router_output_req[dir], nb);
		}
	}

	~UnitTest_SHMACRouterOutputPort()
	{
		sc_close_vcd_trace_file(Tf);
		delete uut;
		delete clk;
	}

	void waitClockPosedge(int n = 1)
	{
		while (n)
		{
			n--;
			wait(clk->posedge_event());
		}
	}

	void run()
	{
		shmacsim::packet testPacket;
		testPacket.data = 0xcafebabe;
		testPacket.req = true;
		testPacket.sender =
		{	1,1};
		testPacket.write_enable = shmacsim::PACKET_WREN_WRITE4;
		testPacket.flag = shmacsim::PACKET_FLAG_NONE;
		testPacket.address = 0xdeadbeef;

		waitClockPosedge();
		//sc_assert(arbiter_enable.read() == true);

		grant_valid = true;
		grant = shmacsim::DIR_NORTH;
		SHMAC_FOREACH_DIR(dir)
			router_output_req[dir] = (dir == shmacsim::DIR_NORTH ? true : false);
		router_output_data.write(testPacket);

		wait(arbiter_enable.negedge_event());

		grant_valid = false;

		SHMAC_FOREACH_DIR(dir)
			router_output_req[dir] = false;

		wait(tx_req.posedge_event());

		sc_assert(tx_data.read().data == 0xcafebabe);
		std::cout << "UnitTest_SHMACRouterOutputPort: verified data consistency"
				<< std::endl;

		tx_ack = true;

		wait(tx_req.negedge_event());

		tx_ack = false;

		std::cout << "Unit test SHMACRouterOutputPort passed!" << std::endl;
	}
};

#endif /* UNITTEST_SHMACROUTEROUTPUTPORT_H_ */
