#!/bin/sh

if [ "$#" -lt "3" ]; then
  echo "mips-elf-gcc wrapper script for SHMACsim"
  echo "Usage: shmacsim-archc-compile tile_i_coord tile_j_coord gcc_args"
  echo "Outputs will be named (tile_i, tile_j) in .x (ELF executable) and .bin (raw binary) formats"
  exit 0
fi

SHMACSIM_TOOLS_DIR=$SHMACSIM_ROOT/tools
ARCHC_LDSCRIPT_GEN_NAME=create_archc_linkerscript.sh
SHMACSIM_SPEC_NAME=shmacsim_specs

MEM_LENGTH=0x1000000
MEM_ORIGIN=0x"$1$2"000000
CMDLINE_ARGS_RESERVE=1024
PROG_NAME="($1","$2)"

# remove any old linker scripts, suppress errors
rm $SHMACSIM_TOOLS_DIR/temp.ld 2> /dev/null

echo "Compiling for tile:\t($1,$2)"
echo "Tile RAM start:\t\t$MEM_ORIGIN"
echo "Tile RAM size:\t\t$MEM_LENGTH"
echo "Cmdline args resv:\t$CMDLINE_ARGS_RESERVE"

# create ArchC linker script for given configuration
# this linker script will place sections into the local tile RAM
$SHMACSIM_TOOLS_DIR/$ARCHC_LDSCRIPT_GEN_NAME $MEM_ORIGIN $MEM_LENGTH $CMDLINE_ARGS_RESERVE > $SHMACSIM_TOOLS_DIR/temp.ld

# shift out the first 3 arguments, we'll past the rest to gcc
shift 2

# call GCC with the rest of the arguments
mips-elf-gcc -specs=$SHMACSIM_TOOLS_DIR/$SHMACSIM_SPEC_NAME -T$SHMACSIM_TOOLS_DIR/temp.ld $@ -o $PROG_NAME.x

# call objcopy to generate binary output
# keep the .bss section in generated output
mips-elf-objcopy -K .bss --set-section-flags .bss=alloc,load,contents -I elf32-bigmips -O binary $PROG_NAME.x $PROG_NAME.bin

# remove any old linker scripts, suppress errors
rm $SHMACSIM_TOOLS_DIR/temp.ld 2> /dev/null
