#!/bin/sh

# Location of SHMACsim executable
SHMACSIM_PATH="$SHMACSIM_ROOT/shmacsim/Debug/shmacsim"

# Benchmark folder configration, each benchmark
# $BENCHMARK is assumed to be structured as:
# hardware config file at $BENCHMARKS_ROOT/$BENCHMARK/$HARDWARECFG_FILENAME
# runtime config file at $BENCHMARKS_ROOT/$BENCHMARK/$RUNTIMECFG_FILENAME
BENCHMARKS_ROOT="$SHMACSIM_ROOT/microbenchmarks"
HARDWARECFG_FILENAME="hardwarecfg.txt"
RUNTIMECFG_FILENAME="runtimecfg.txt"

# The stdout and stderr from each benchmark will be
# placed in the BENCHMARK_LOGDIIR, whose name is
# generated using the current date-time
DATE=$(date +"%Y%m%d%H%M%S")
BENCHMARK_LOGDIR="$SHMACSIM_ROOT/benchmark_logs/$DATE"

BENCHMARKS=$@


echo "The following SHMACsim benchmarks will be run: "
echo $BENCHMARKS
mkdir -p $BENCHMARK_LOGDIR
echo "Benchmark results will be placed in $BENCHMARK_LOGDIR"

export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$SYSTEMC/lib-linux64:$SYSTEMC/lib-linux

PROCESS_LIST="$$"

for benchmark in $BENCHMARKS; 
do
	echo "Now launching benchmark: $benchmark"
	BENCHMARK_SUBDIR=$(dirname $BENCHMARK_LOGDIR/$benchmark)
	mkdir -p $BENCHMARK_SUBDIR
	$SHMACSIM_PATH -h $BENCHMARKS_ROOT/$benchmark/$HARDWARECFG_FILENAME -r $BENCHMARKS_ROOT/$benchmark/$RUNTIMECFG_FILENAME 1>"$BENCHMARK_LOGDIR/$benchmark"_out.log 2>"$BENCHMARK_LOGDIR/$benchmark"_err.log &
	PROCESS_LIST="$PROCESS_LIST,$!"
done

echo "Use the following command to monitor benchmark process status:"
echo "top -p $PROCESS_LIST"
